@@ -60,9 +60,10 @@ sub update_job_deps {
 
     return if $self->use_batches;
 
-    return unless $self->current_batch->has_array_deps;
+    # return unless $self->current_batch->has_array_deps;
+    return unless $self->has_array_deps;
 
-    foreach my $array_id ( $self->current_batch->all_array_deps ) {
+    foreach my $array_id ( $self->all_array_deps ) {
         next unless $array_id;
 
         my $current_job = $array_id->[0];
@@ -70,6 +71,7 @@ sub update_job_deps {
 
         my $cmd =
           "scontrol update job=$current_job Dependency=afterok:$dep_job";
+
         $self->app_log->warn($cmd);
     }
 }
